@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v0.2.1
+## Version v1.0.0
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
-## Modified 2021/05/12
+## Modified 2021/05/14
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -20,9 +20,10 @@ BEGIN
     use IO::File;
     use PPI;
     use Filter::Util::Call;
-    use Scalar::Util;
+    use Scalar::Util ();
     use List::Util ();
-    our $VERSION = 'v0.2.1';
+    use Want ();
+    our $VERSION = 'v1.0.0';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -33,10 +34,19 @@ our $SENTINEL = bless( {} => __PACKAGE__ . '::SENTINEL' );
 sub import
 {
     my( $this, @arguments ) = @_ ;
+    my $class = caller();
     my $hash = { @arguments };
     $hash->{debug} = 0 if( !CORE::exists( $hash->{debug} ) );
     $hash->{no_filter} = 0 if( !CORE::exists( $hash->{no_filter} ) );
     $hash->{debug_code} = 0 if( !CORE::exists( $hash->{debug_code} ) );
+    # We check if we are running under tie and if so we cannot use Want features, 
+    # because they would trigger a segmentation fault.
+    $hash->{is_tied} = 0;
+    if( $class->can( 'TIESCALAR' ) || $class->can( 'TIEHASH' ) || $class->can( 'TIEARRAY' ) )
+    {
+        $hash->{is_tied} = 1;
+    }
+    *{"${class}::caller"} = \&{"Nice::Try::caller"};
     filter_add( bless( $hash => ( ref( $this ) || $this ) ) );
 }
 
@@ -45,6 +55,13 @@ sub unimport
     filter_del();
 }
 
+sub caller(;$)
+{
+    my $n = shift( @_ );
+    my @info = defined( $n ) ? CORE::caller( int( $n ) + 2 ) : CORE::caller(3);
+    return( @info );
+}
+
 sub filter
 {
     my( $self ) = @_ ;
@@ -585,6 +602,7 @@ EOT
             $try_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
             
             my $try_sub = <<EOT;
+CORE::local \$Nice::Try::WANT;
 CORE::local \$Nice::Try::TRY = CORE::sub
 {
     CORE::do __TRY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ };__TRY__CLOSE_NL__
@@ -593,21 +611,104 @@ CORE::local \$Nice::Try::TRY = CORE::sub
 CORE::local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL, \@Nice::Try::VOID );
 __FINALLY_BLOCK__ CORE::local \$Nice::Try::HAS_CATCH = $has_catch_clause;
 CORE::local \$Nice::Try::WANTARRAY = CORE::wantarray;
+EOT
+            if( !$self->{is_tied} )
+            {
+                $try_sub .= <<EOT;
+CORE::local \$Nice::Try::NOOP = sub
+{
+    my \$ref = CORE::shift( \@_ );
+    CORE::return(sub{ CORE::return( \$ref ) });
+};
+if( CORE::defined( \$Nice::Try::WANTARRAY ) )
+{
+    eval "\\\$Nice::Try::WANT = Want::want( 'LIST' )
+            ? 'LIST'
+            : Want::want( 'HASH' )
+                ? 'HASH'
+                : Want::want( 'ARRAY' )
+                    ? 'ARRAY'
+                    : Want::want( 'OBJECT' )
+                        ? 'OBJECT'
+                        : Want::want( 'CODE' )
+                            ? 'CODE'
+                            : Want::want( 'REFSCALAR' )
+                                ? 'REFSCALAR'
+                                : Want::want( 'BOOLEAN' )
+                                    ? 'BOOLEAN'
+                                    : Want::want( 'GLOB' )
+                                        ? 'GLOB'
+                                        : Want::want( 'SCALAR' )
+                                            ? 'SCALAR'
+                                            : Want::want( 'VOID' )
+                                                ? 'VOID'
+                                                : '';";
+    undef( \$Nice::Try::WANT ) if( \$\@ );
+}
+EOT
+            }
+            $try_sub .= <<EOT;
 {
     CORE::local \$\@;
     CORE::eval 
     {
-        if( \$Nice::Try::WANTARRAY ) 
+        if( CORE::defined( \$Nice::Try::WANT ) && CORE::length( \$Nice::Try::WANT ) )
         {
-            \@Nice::Try::RETVAL = &\$Nice::Try::TRY;
+            if( \$Nice::Try::WANT eq 'OBJECT' )
+            {
+                \$Nice::Try::RETVAL[0] = Nice::Try::ObjectContext->new( &\$Nice::Try::TRY )->callback();
+            }
+            elsif( \$Nice::Try::WANT eq 'CODE' )
+            {
+                \$Nice::Try::RETVAL[0] = \$Nice::Try::NOOP->( &\$Nice::Try::TRY )->();
+            }
+            elsif( \$Nice::Try::WANT eq 'HASH' )
+            {
+                \@Nice::Try::RETVAL = \%{ &\$Nice::Try::TRY };
+            }
+            elsif( \$Nice::Try::WANT eq 'ARRAY' )
+            {
+                \@Nice::Try::RETVAL = \@{ &\$Nice::Try::TRY };
+            }
+            elsif( \$Nice::Try::WANT eq 'REFSCALAR' )
+            {
+                \$Nice::Try::RETVAL[0] = \${&\$Nice::Try::TRY};
+            }
+            elsif( \$Nice::Try::WANT eq 'GLOB' )
+            {
+                \$Nice::Try::RETVAL[0] = \*{ &\$Nice::Try::TRY };
+            }
+            elsif( \$Nice::Try::WANT eq 'LIST' )
+            {
+                \@Nice::Try::RETVAL = &\$Nice::Try::TRY;
+            }
+            elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
+            {
+                \$Nice::Try::RETVAL[0] = &\$Nice::Try::TRY ? 1 : 0;
+            }
+            elsif( \$Nice::Try::WANT eq 'VOID' )
+            {
+                \$Nice::Try::VOID[0] = &\$Nice::Try::TRY;
+            }
+            elsif( \$Nice::Try::WANT eq 'SCALAR' )
+            {
+                \$Nice::Try::RETVAL[0] = &\$Nice::Try::TRY;
+            }
         }
-        elsif( defined( \$Nice::Try::WANTARRAY ) ) 
+        else
         {
-            \$Nice::Try::RETVAL[0] = &\$Nice::Try::TRY;
-        }
-        else 
-        {
-            \@Nice::Try::VOID = &\$Nice::Try::TRY;
+            if( \$Nice::Try::WANTARRAY ) 
+            {
+                \@Nice::Try::RETVAL = &\$Nice::Try::TRY;
+            }
+            elsif( defined( \$Nice::Try::WANTARRAY ) ) 
+            {
+                \$Nice::Try::RETVAL[0] = &\$Nice::Try::TRY;
+            }
+            else 
+            {
+                \@Nice::Try::VOID = &\$Nice::Try::TRY;
+            }
         }
     };
     \$Nice::Try::DIED = CORE::length( \$\@ ) ? 1 : 0;
@@ -737,17 +838,63 @@ EOT
                 CORE::return \$Nice::Try::SENTINEL;
             };
             
-            if( \$Nice::Try::WANTARRAY ) 
+            if( CORE::defined( \$Nice::Try::WANT ) && CORE::length( \$Nice::Try::WANT ) )
             {
-                \@Nice::Try::RETVAL = \&\$Nice::Try::CATCH;
+                if( \$Nice::Try::WANT eq 'OBJECT' )
+                {
+                    \$Nice::Try::RETVAL[0] = Nice::Try::ObjectContext->new( \&\$Nice::Try::CATCH )->callback();
+                }
+                elsif( \$Nice::Try::WANT eq 'CODE' )
+                {
+                    \$Nice::Try::RETVAL[0] = \$Nice::Try::NOOP->( \&\$Nice::Try::CATCH )->();
+                }
+                elsif( \$Nice::Try::WANT eq 'HASH' )
+                {
+                    \@Nice::Try::RETVAL = \%{ \&\$Nice::Try::CATCH };
+                }
+                elsif( \$Nice::Try::WANT eq 'ARRAY' )
+                {
+                    \@Nice::Try::RETVAL = \@{ \&\$Nice::Try::CATCH };
+                }
+                elsif( \$Nice::Try::WANT eq 'REFSCALAR' )
+                {
+                    \$Nice::Try::RETVAL[0] = \${\&\$Nice::Try::CATCH};
+                }
+                elsif( \$Nice::Try::WANT eq 'GLOB' )
+                {
+                    \$Nice::Try::RETVAL[0] = \*{ \&\$Nice::Try::CATCH };
+                }
+                elsif( \$Nice::Try::WANT eq 'LIST' )
+                {
+                    \@Nice::Try::RETVAL = \&\$Nice::Try::CATCH;
+                }
+                elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
+                {
+                    \$Nice::Try::RETVAL[0] = \&\$Nice::Try::CATCH ? 1 : 0;
+                }
+                elsif( \$Nice::Try::WANT eq 'VOID' )
+                {
+                    \$Nice::Try::VOID[0] = \&\$Nice::Try::CATCH;
+                }
+                elsif( \$Nice::Try::WANT eq 'SCALAR' )
+                {
+                    \$Nice::Try::RETVAL[0] = \&\$Nice::Try::CATCH;
+                }
             }
-            elsif( defined( \$Nice::Try::WANTARRAY ) )
+            else
             {
-                \$Nice::Try::RETVAL[0] = \&\$Nice::Try::CATCH;
-            } 
-            else 
-            {
-                \@Nice::Try::VOID = \&\$Nice::Try::CATCH;
+                if( \$Nice::Try::WANTARRAY ) 
+                {
+                    \@Nice::Try::RETVAL = \&\$Nice::Try::CATCH;
+                }
+                elsif( defined( \$Nice::Try::WANTARRAY ) )
+                {
+                    \$Nice::Try::RETVAL[0] = \&\$Nice::Try::CATCH;
+                } 
+                else 
+                {
+                    \@Nice::Try::VOID = \&\$Nice::Try::CATCH;
+                }
             }
 EOT
                 if( $cdef->{var} )
@@ -897,6 +1044,60 @@ if( CORE::defined( \$Nice::Try::WANTARRAY ) and
     {
         \$Nice::Try::BREAK = 'redo';
     }
+    elsif( CORE::defined( \$Nice::Try::WANT ) && CORE::length( \$Nice::Try::WANT ) )
+    {
+        if( \$Nice::Try::WANT eq 'LIST' )
+        {
+            CORE::return( \@Nice::Try::RETVAL );
+        }
+        elsif( \$Nice::Try::WANT eq 'VOID' )
+        {
+            if( \$Nice::Try::RETVAL[0] eq '__NEXT__' )
+            {
+                \$Nice::Try::BREAK = 'next';
+            }
+            elsif( \$Nice::Try::RETVAL[0] eq '__LAST__' )
+            {
+                \$Nice::Try::BREAK = 'last';
+            }
+            elsif( \$Nice::Try::RETVAL[0] eq '__REDO__' )
+            {
+                \$Nice::Try::BREAK = 'redo';
+            }
+        }
+        elsif( \$Nice::Try::WANT eq 'OBJECT' )
+        {
+            CORE::return( \$Nice::Try::RETVAL[0] );
+        }
+        elsif( \$Nice::Try::WANT eq 'REFSCALAR' )
+        {
+            CORE::return( \\\$Nice::Try::RETVAL[0] );
+        }
+        elsif( \$Nice::Try::WANT eq 'SCALAR' )
+        {
+            CORE::return( \$Nice::Try::RETVAL[0] );
+        }
+        elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
+        {
+            CORE::return( \$Nice::Try::RETVAL[0] );
+        }
+        elsif( \$Nice::Try::WANT eq 'CODE' )
+        {
+            CORE::return( \$Nice::Try::RETVAL[0] );
+        }
+        elsif( \$Nice::Try::WANT eq 'HASH' )
+        {
+            CORE::return( { \@Nice::Try::RETVAL } );
+        }
+        elsif( \$Nice::Try::WANT eq 'ARRAY' )
+        {
+            CORE::return( \\\@Nice::Try::RETVAL );
+        }
+        elsif( \$Nice::Try::WANT eq 'GLOB' )
+        {
+            CORE::return( \$Nice::Try::RETVAL[0] );
+        }
+    }
     else
     {
         CORE::return( \$Nice::Try::WANTARRAY ? \@Nice::Try::RETVAL : \$Nice::Try::RETVAL[0] );
@@ -1184,8 +1385,8 @@ sub _serialize
 
 
 {
-  package # hide from PAUSE
-    Nice::Try::ScopeGuard;
+    package # hide from PAUSE
+        Nice::Try::ScopeGuard;
 
     # older versions of perl have an issue with $@ during global destruction
     use constant UNSTABLE_DOLLARAT => ("$]" < '5.013002') ? 1 : 0;
@@ -1225,10 +1426,30 @@ sub _serialize
     }
 }
 
+{
+    package
+        Nice::Try::ObjectContext;
+
+    sub new
+    {
+        my $that = shift( @_ );
+        # print( STDERR "Got here in Nice::Try::ObjectContext->new with args '", join( "', '", @_ ), "'\n" );
+        return( bless( { val => [@_] } => ( ref( $that ) || $that ) ) );
+    }
+
+    sub callback
+    {
+        my $self = shift( @_ );
+        # print( STDERR "Got here in Nice::Try::ObjectContext->dummy with args '", join( "', '", @_ ), "'\n" );
+        return( $self->{val}->[0] );
+    }
+}
+
 {
     package
         PPI::Element;
     
+    no warnings 'redefine';
     sub replace {
         my $self    = ref $_[0] ? shift : return undef;
         # If our object and the other are not of the same class, PPI refuses to replace 
@@ -1289,9 +1510,73 @@ When run, this would produce, as one would expect:
     Cleaning up
     Ok, then
 
+Also since version 1.0.0, L<Nice::Try> is context aware:
+
+    use Want; # an awesome module which extends wantarray
+    sub info
+    {
+        my $self = shift( @_ );
+        try
+        {
+            # Do something
+            if( want('OBJECT') )
+            {
+                return( $self );
+            }
+            elsif( want('CODE') )
+            {
+                # dummy code ref for example
+                return( sub{ return( $name ); } );
+            }
+            elsif( want('LIST') )
+            {
+                return( @some_data );
+            }
+            elsif( want('ARRAY') )
+            {
+                return( \@some_data );
+            }
+            elsif( want('HASH') )
+            {
+                return({ name => $name, location => $city });
+            }
+            elsif( want('REFSCALAR') )
+            {
+                return( \$name );
+            }
+            elsif( want('SCALAR' ) )
+            {
+                return( $name ); # regular string
+            }
+            elsif( want('VOID') )
+            {
+                return;
+            }
+        }
+        catch( $e )
+        {
+            $Logger->( "Caught exception: $e" );
+        }
+    }
+
+    # regular string context
+    my $name = $o->info;
+    # code context
+    my $name = $o->info->();
+    # list context like wantarray
+    my @data = $o->info;
+    # hash context
+    my $name = $o->info->{name};
+    # array context
+    my $name = $o->info->[2];
+    # object context
+    my $name = $o->info->another_method;
+    # scalar reference context
+    my $name = ${$o->info};
+
 =head1 VERSION
 
-    v0.2.1
+    v1.0.0
 
 =head1 DESCRIPTION
 
@@ -1429,6 +1714,37 @@ Would produce:
 
 =item * Supports a C<finally> block called in void context for cleanup for example
 
+=item * L<Nice::Try> is rich context aware, which means it can provide you with a super granual context on how to return data back to the caller based on the caller's expectation, by using a module like L<Want>.
+
+=item * Call to L<perlfunc/caller> will return the correct entry in call stack
+
+    #!/usr/bin/perl
+    BEGIN
+    {
+        use strict;
+        use warnings;
+        use Nice::Try;
+    };
+
+    {
+        &callme();
+    }
+
+    sub callme
+    {
+        try
+        {
+            my @info = caller(1); # or my @info = caller;
+            print( "Called from package $info[0] in file $info[1] at line $info[2]\n" );
+        }
+        catch( $e )
+        {
+            print( "Got an error: $e\n" );
+        }
+    }
+
+WIll yield: C<Called from package main in file ./test.pl at line 10>
+
 =back
 
 =head1 WHY USE IT?
@@ -1509,7 +1825,46 @@ will produces:
 
 In group 3, L<TryCatch> was working wonderfully, but was relying on L<Devel::Declare> which was doing some esoteric stuff and eventually the version 0.006020 broke L<TryCatch> and there seems to be no intention of correcting this breaking change.
 
-In group 4, there is L<Syntax::Keyword::Try>, which is a great alternative if you do not care about exception variable assignment or exception class filter. You can only use C<$@>
+In group 4, there is L<Syntax::Keyword::Try>, which is a great alternative if you do not care about exception class filter (it supports variable assignment since 2020-08-01 with version 0.18).
+
+Although, the following script would not work under L<Syntax::Keyword::Try> :
+
+    BEGIN
+    {
+        use strict;
+        use warnings;
+        use Syntax::Keyword::Try;
+    };
+
+    {
+        &callme();
+    }
+
+    sub callme
+    {
+        try {
+            print( "Hello there\n" );
+        }
+        catch ($e) {
+            print( "Got an error: $e\n" );
+        }
+    }
+
+This will trigger the following error:
+
+    syntax error at ./test.pl line 18, near ") {"
+    syntax error at ./test.pl line 21, near "}"
+    Execution of ./test.pl aborted due to compilation errors.
+
+That is because L<Syntax::Keyword::Try> expects to be C<used> outside of a BEGIN block like this:
+
+    use strict;
+    use warnings;
+    use Syntax::Keyword::Try;
+
+    # Rest of the script, same as above
+
+Of course, with L<Nice::Try>, there is no such constraint. You can L<perlfunc/use> L<Nice::Try> inside or outside of a C<BEGIN> block indistinctively.
 
 Since L<perl version 5.33.7|https://perldoc.perl.org/blead/perlsyn#Try-Catch-Exception-Handling> you can use the try-catch block using an experimental feature which may be removed in future versions, by writing:
 
@@ -1728,6 +2083,125 @@ However, if you enclose a try-catch block inside another block, use of C<next>,
         }
     }
 
+=head1 CONTEXT AWARENESS
+
+L<Nice::Try> provides a high level of granularity about the context in which your subroutine was called.
+
+Normally, you would write something like this, and it works as always:
+
+    sub info
+    {
+        try
+        {
+            # do something here
+            if( wantarray() )
+            {
+                return( @list_of_values );
+            }
+            # caller just want a scalar
+            elsif( defined( wantarray() ) )
+            {
+                return( $name );
+            }
+            # otherwise if undefined, it means we are called in void context, like:
+            # $o->info; with no expectation of return value
+        }
+        catch( $e )
+        {
+            print( "Caught an error: $e\n" );
+        }
+    }
+
+THe above is nice, but how do you differentiate cases were your caller wants a simple returned value and the one where the caller wants an object for chaining purpose, or if the caller wants an hash or array reference in return?
+
+For example:
+
+    my $val = $o->info->[2]; # wants an array reference
+    my $val = $o->info->{name} # wants an hash reference
+    # etc...
+
+Now, you can do the following:
+
+    use Want; # an awesome module which extends wantarray
+    sub info
+    {
+        my $self = shift( @_ );
+        try
+        {
+            # Do something
+            # 
+            # same as wantarray() == 1
+            if( want('LIST') )
+            {
+                return( @some_data );
+            }
+            # same as: if( defined( wantarray() ) && !wantarray() )
+            elsif( want('SCALAR' ) )
+            {
+                return( $name ); # regular string
+            }
+            # same as !defined( wantarray() )
+            elsif( want('VOID') )
+            {
+                return;
+            }
+            # For the other context below, wantarray is of no help
+            if( want('OBJECT') )
+            {
+                return( $obj ); # useful for chaining
+            }
+            elsif( want('CODE') )
+            {
+                # dummy code ref for example
+                return( sub{ return( $name ); } );
+            }
+            elsif( want('ARRAY') )
+            {
+                return( \@some_data );
+            }
+            elsif( want('HASH') )
+            {
+                return({ name => $name, location => $city });
+            }
+        }
+        catch( $e )
+        {
+            $Logger->( "Caught exception: $e" );
+        }
+    }
+
+Thus this is particularly useful if, for example, you want to differentiate if the caller just wants a return string, or an object for chaining.
+
+L<perlfunc/wantarray> would not know the difference, and other try-catch implementation would not let you benefit from using L<Want>.
+
+For example:
+
+    my $val = $o->info; # simple regular scalar context; but...
+    # here, we are called in object context and wantarray is of no help to tell the difference
+    my $val = $o->info->another_method;
+
+Other cases are:
+
+    # regular string context
+    my $name = $o->info;
+    # list context like wantarray
+    my @data = $o->info;
+
+    # code context
+    my $name = $o->info->();
+    # hash context
+    my $name = $o->info->{name};
+    # array context
+    my $name = $o->info->[2];
+    # object context
+    my $name = $o->info->another_method;
+
+See L<Want> for more information on how you can benefit from it.
+
+Currently lvalues are no implemented and will be in future releases.
+
+Also, for this rich context awareness to be used, obviously try-catch would need to be inside a subroutine, otherwise there is no rich context other than the one the regular L<perlfunc/wantarray> provides.
+
 =head1 DEBUGGING
 
 And to have L<Nice::Try> save the filtered code to a file, pass it the C<debug_file> parameter like this:
